(*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON-Web.
 *
 * UMLMON-Web is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON-Web is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON-Web; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 *)

(* $Id: termin.ml 121 2005-06-12 13:56:40Z gerd $
 * ----------------------------------------------------------------------
 *
 *)

open Wd_dialog
open Wd_run_cgi
open Wd_types
open Netcgi_types
open Definitions

open Umlmonctrl_aux
open Printf

module Mon = Umlmonctrl_clnt.Ctrl.V1


let translate_line s =
  let l = ref [] in
  for k = 0 to String.length s - 1 do
    l := (`key_translated (Int32.of_int (Char.code s.[k]))) :: !l
  done;
  List.rev !l


let code_re = Str.regexp "code\\([0-9]+\\)$"

let fn_re = Str.regexp "fn\\([0-9]+\\)$"

let translate_special s =
  if Str.string_match code_re s 0 then
    [`key_translated (Int32.of_string (Str.matched_group 1 s))]
  else
    if Str.string_match fn_re s 0 then
      [`key_fn (Int32.of_string (Str.matched_group 1 s))]
    else
      []


class termin universe name env =
object (self)
  inherit dialog universe name env
  inherit std_behaviour


  method private try_prepare_page() =
    let cgi = self # environment . cgi in
    if cgi # argument_value "id" <> "" then (
      self # set_variable "id" (String_value (cgi # argument_value "id"));
      self # set_variable "password" (String_value (cgi # argument_value "password"));
      self # set_variable "ct_name" (String_value (cgi # argument_value "ct_name"));
      self # set_variable "ct_id" (String_value (cgi # argument_value "ct_id"));
    );
    self # set_variable "input_special" (Enum_value []);
    self # set_variable "input_line" (String_value "");

  method private try_handle() =
    try
      let final_keys =
      match self # event with
	| Image_button("enter",_,_) -> [`key_enter]
	| Image_button("submit",_,_) ->  []
	| Image_button("left",_,_) -> [ `key_cursor_left ]
	| Image_button("right",_,_) -> [ `key_cursor_right ]
	| Image_button("up",_,_) -> [ `key_cursor_up ]
	| Image_button("down",_,_) -> [ `key_cursor_down ]
	| Image_button("backspace",_,_) -> [ `key_backspace ]
	| Image_button("del",_,_) -> [ `key_delete ]
	| Image_button("ins",_,_) -> [ `key_insert ]
	| Image_button("ctrl_c",_,_) -> [ `key_translated 3l ]
	| Image_button("home",_,_) -> [ `key_home ]
	| Image_button("end",_,_) -> [ `key_end ]
	| Image_button("pg_up",_,_) -> [ `key_page_up ]
	| Image_button("pg_down",_,_) -> [ `key_page_down ]
	| _ -> []
      in
      let id = self # string_variable "id" in
      if id="" then raise Not_found;
      let ct_name = self # string_variable "ct_name" in
      let ct_id = self # string_variable "ct_id" in
      let handle = { ct_name = ct_name; ct_id = Int32.of_string ct_id } in
      let line_keys = translate_line (self # string_variable "input_line") in
      let special_keys = 
	List.flatten(List.map translate_special (self # enum_variable "input_special")) in
      let keys = line_keys @ special_keys @ final_keys in
      Common.call_simple_proc 
	id self#creds Mon.te_write_keys (handle, Array.of_list keys, 5l)
    with
	Not_found -> ()

  method private check_auth () = ()
end
;;


Registry.new_termin :=
  fun universe env ->
    let dlg = universe # create env "termin" in
    dlg
;;
